"""GMailAuthPlugin
Copyright(C), 2005, Enfold Systems, LLC - ALL RIGHTS RESERVED

This software is licensed under the Terms and Conditions contained within the
LICENSE.txt file that accompanied this software.  Any inquiries concerning the
scope or enforceability of the license should be addressed to:

Enfold Systems, LLC
4500 Montrose Blvd., Suite D
Houston, Texas 77006 USA
p. +1 713.942.2377 | f. +1 832.201.8856
www.enfoldsystems.com
info@enfoldsystems.com
"""

from AccessControl import ClassSecurityInfo

from Globals import InitializeClass
from OFS.Cache import Cacheable

from Products.PluggableAuthService.plugins.BasePlugin import BasePlugin
from Products.PluggableAuthService.utils import classImplements
from Products.PluggableAuthService.interfaces.plugins import IAuthenticationPlugin

from zLOG import LOG
try:
    # use system libgmail if installed
    import libgmail
    LOG("GMailAuthPlugin",0,"using system-installed libgmail")
except ImportError:
    # otherwise, use local
    from libgmail_incl import libgmail

from Products.PageTemplates.PageTemplateFile import PageTemplateFile

manage_addGMailAuthPluginForm = PageTemplateFile(
    'www/gmailAdd', globals(), __name__='manage_addGMailAuthPluginForm' )

def manage_addGMailAuthPlugin(dispatcher, id, title=None, REQUEST=None):
    """ Add a GMailAuthPlugin to a Pluggable Auth Service. """

    obj = GMailAuthPlugin(id, title)
    dispatcher._setObject(obj.getId(), obj)

    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect(
                                '%s/manage_workspace'
                                '?manage_tabs_message='
                                'GMailAuthPlugin+added.'
                            % dispatcher.absolute_url())

class GMailAuthPlugin(BasePlugin, Cacheable):

    """ PAS plugin for using GMail credentials to log in.
    """

    meta_type = 'GMailAuthPlugin'

    security = ClassSecurityInfo()

    def __init__(self, id, title=None):
        self._id = self.id = id
        self.title = title


    #
    #   IAuthenticationPlugin implementation
    #
    security.declarePrivate('authenticateCredentials')
    def authenticateCredentials(self, credentials):

        """ See IAuthenticationPlugin.

        o We expect the credentials to be those returned by
          ILoginPasswordExtractionPlugin.
        """
        login = credentials.get( 'login' )
        password = credentials.get( 'password' )

        if login is None or password is None:
            return None

        # it will accept username or username@gmail.com
        # we should probably be normailizing the login,
        # but I don't know if there are other domains you can use in gmail--
        # probably not.
        ga = libgmail.GmailAccount(login, password)
        try:
            ga.login()
        except libgmail.GmailLoginFailure:
            return None

        return ga.name, ga.name


classImplements(GMailAuthPlugin, IAuthenticationPlugin)

InitializeClass(GMailAuthPlugin)
